#include <iostream>  // std::cout
#include <list>      // std::list
#include <algorithm> // std::copy

#include "statistics.hpp" // boost::statistics

template< typename T >
void display_results( const boost::statistics< T > & stats )
{
   std::cout << "sum(x)    = " << stats.sumX()   << '\n';
   std::cout << "count(x)  = " << stats.count()  << '\n';
   std::cout << "x-bar     = " << stats.xbar()   << '\n';
   std::cout << "x^2-bar   = " << stats.x2bar()  << '\n';
   std::cout << "stddev(x) = " << stats.stddev() << '\n';
   std::cout << "var(x)    = " << stats.var()    << '\n';
}

int __cdecl main()
{
   std::cout << "statistics test" << '\n';

   boost::statistics< double >         stats;

   // compute test

   std::cout << '\n' << "compute( T & const ):" << '\n';

   double                              data[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };

   for( int i = 0; i < 6; ++i )        stats.compute( data[ i ]);
   display_results( stats );

   // iterator test

   std::cout << '\n' << "iterator test:" << '\n';
   stats.reset();

   std::list< double >                 data2;
   for( int i = 0; i < 6; ++i )        data2.push_back( data[ i ] * 5 );

   std::copy( data2.begin(), data2.end(), stats.begin());
   display_results( stats );

   // container operations

   std::cout << '\n' << "container operations:" << '\n';
   stats.reset();

   stats.push_front( 10 );
   stats.push_back(  20 );
   stats.insert( stats.begin(), 30 );

   display_results( stats );

   std::cout << '\n' << "complex statistics:" << '\n';

   boost::statistics< std::complex< float > >
                                       cstats;

   cstats.push_front( std::complex< float >(  1.0,  0.0 ));
   cstats.push_front( std::complex< float >(  2.0,  0.0 ));
   cstats.push_front( std::complex< float >(  7.9,  2.7 ));
   cstats.push_front( std::complex< float >( 37.0, -2.2 ));
   display_results( cstats );

   return( 0 );
}

